%--------------------------------------------------------------------------
%
% Generate the electric field (E) errors on the grid for Vacuum Patch Test
%
%   INPUT: ABC contains the complex E field in the formnat define 
%          in Annex C; indexes (i,j,k) start at 0
%   OUTPUT: rmsErr(1:3): rms errors for the three field components(Ex,Ey,Ez)
%           maxErr(1:3): maximum errors for the three field components
%           oriErr(1:3): error at the origin
%
%           All E field errors are normalized to E0 . (The
%           modulus of the incident E field.)
%
%--------------------------------------------------------------------------

function [ rmsErr, maxErr, oriErr ] = computeEfieldErrorInVacuumCube( params, ABC )

siz = size(ABC);
M = siz(1);

E0 = params.E0;
k0 = params.k0;

% calculate point errors for E components
err = zeros(M,3);
mOri = 0;
minR = 1E30;
for m = 1:M
    % get calculated field
    Ex = complex(ABC(m,7), ABC(m,8));
    Ey = complex(ABC(m,9), ABC(m,10));
    Ez = complex(ABC(m,11), ABC(m,12));
    
    % compute incident field
    y = ABC(m,5);
    jk0y = complex(0, k0*y);
    exactEx = complex(0,0);
    exactEy = complex(0,0);
    exactEz = E0 * exp(-jk0y);

    % compute point error
    err(m,1) = abs(Ex-exactEx)/E0;
    err(m,2) = abs(Ey-exactEy)/E0;
    err(m,3) = abs(Ez-exactEz)/E0;
    
    % store origin index
    x = ABC(m,4);
    z = ABC(m,6);
    R = sqrt(x^2 + y^2 + z^2);
    if R < minR 
        minR = R;
        mOri = m;
    end
end

% calculate rms error
rmsErr = [0; 0; 0];
for m = 1:M
    for c=1:3
        rmsErr(c) = rmsErr(c) + err(m,c)^2;
    end
end
for c=1:3
    rmsErr(c) = sqrt( rmsErr(c)/M );
end

% calculate maximum error and the location of maximum error in Ez
maxErr = [0; 0; 0];
maxLoc = [0; 0; 0];
for m = 1:M
    if err(m,1) > maxErr(1)
        maxErr(1) = err(m,1);
    end
    if err(m,2) > maxErr(2)
        maxErr(2) = err(m,2);
    end
    if err(m,3) > maxErr(3)
        maxErr(3) = err(m,3);
        % store location
        maxLoc(1) = ABC(m,4);
        maxLoc(2) = ABC(m,5);
        maxLoc(3) = ABC(m,6);
    end
end

% calculate error at origin
oriErr = [0; 0; 0];
if mOri ~= 0
    for c=1:3
        oriErr(c) = err(mOri,c);
    end
end

end

